@@ -2,7 +2,7 @@ include("script/campaign/libcampaign.js");
 include("script/campaign/templates.js");
 
 const mis_collectiveRes = [
-	"R-Defense-WallUpgrade06", "R-Struc-Materials06", "R-Sys-Engineering02",
+	"R-Defense-WallUpgrade05", "R-Struc-Materials05", "R-Sys-Engineering02",
 	"R-Vehicle-Engine04", "R-Vehicle-Metals04", "R-Cyborg-Metals04",
 	"R-Wpn-Cannon-Accuracy02", "R-Wpn-Cannon-Damage05",
 	"R-Wpn-Cannon-ROF01", "R-Wpn-Flamer-Damage06", "R-Wpn-Flamer-ROF01",
@@ -12,6 +12,17 @@ const mis_collectiveRes = [
 	"R-Wpn-Rocket-ROF03", "R-Wpn-RocketSlow-Accuracy03",
 	"R-Wpn-RocketSlow-Damage05", "R-Sys-Sensor-Upgrade01", "R-Wpn-RocketSlow-ROF01"
 ];
+const mis_collectiveResClassic = [
+	"R-Defense-WallUpgrade03", "R-Struc-Materials04", "R-Struc-Factory-Upgrade04",
+	"R-Struc-VTOLPad-Upgrade01", "R-Vehicle-Engine04", "R-Vehicle-Metals03",
+	"R-Cyborg-Metals03", "R-Vehicle-Armor-Heat02", "R-Cyborg-Armor-Heat02",
+	"R-Wpn-Cannon-Accuracy02", "R-Wpn-Cannon-Damage04", "R-Wpn-Cannon-ROF02",
+	"R-Wpn-Flamer-Damage05", "R-Wpn-Flamer-ROF02", "R-Wpn-MG-Damage05",
+	"R-Wpn-MG-ROF03", "R-Wpn-Mortar-Acc02", "R-Wpn-Mortar-Damage04",
+	"R-Wpn-Mortar-ROF02", "R-Wpn-Rocket-Accuracy02", "R-Wpn-Rocket-Damage05",
+	"R-Wpn-Rocket-ROF03", "R-Wpn-RocketSlow-Accuracy03", "R-Wpn-RocketSlow-Damage04",
+	"R-Wpn-RocketSlow-ROF03", "R-Sys-Sensor-Upgrade01"
+];
 
 camAreaEvent("vtolRemoveZone", function(droid)
 {
@@ -82,9 +93,9 @@ function ambushPlayer()
 	});
 }
 
-function vtolAttack()
+function wave2()
 {
-	const list = [cTempl.colcbv, cTempl.colatv];
+	const list = [cTempl.colatv, cTempl.colatv];
 	const ext = {
 		limit: [4, 4], //paired with list array
 		alternate: true,
@@ -93,6 +104,43 @@ function vtolAttack()
 	camSetVtolData(CAM_THE_COLLECTIVE, "vtolAppearPos", "vtolRemove", list, camChangeOnDiff(camMinutesToMilliseconds(5)), "COCommandCenter", ext);
 }
 
+function wave3()
+{
+	const list = [cTempl.colcbv, cTempl.colcbv];
+	const ext = {
+		limit: [4, 4], //paired with list array
+		alternate: true,
+		altIdx: 0
+	};
+	camSetVtolData(CAM_THE_COLLECTIVE, "vtolAppearPos", "vtolRemove", list, camChangeOnDiff(camMinutesToMilliseconds(5)), "COCommandCenter", ext);
+}
+
+function vtolAttack()
+{
+	if (camClassicMode())
+	{
+		const list = [cTempl.colcbv, cTempl.colatv];
+		const ext = {
+			limit: [4, 4], //paired with list array
+			alternate: true,
+			altIdx: 0
+		};
+		camSetVtolData(CAM_THE_COLLECTIVE, "vtolAppearPos", "vtolRemove", list, camChangeOnDiff(camMinutesToMilliseconds(5)), "COCommandCenter", ext);
+	}
+	else
+	{
+		const list = [cTempl.colpbv, cTempl.colpbv];
+		const ext = {
+			limit: [4, 4], //paired with list array
+			alternate: true,
+			altIdx: 0
+		};
+		camSetVtolData(CAM_THE_COLLECTIVE, "vtolAppearPos", "vtolRemove", list, camChangeOnDiff(camMinutesToMilliseconds(5)), "COCommandCenter", ext);
+		queue("wave2", camChangeOnDiff(camSecondsToMilliseconds(30)));
+		queue("wave3", camChangeOnDiff(camSecondsToMilliseconds(60)));
+	}
+}
+
 function truckDefense()
 {
 	if (enumDroid(CAM_THE_COLLECTIVE, DROID_CONSTRUCT).length === 0)
@@ -109,63 +157,77 @@ function transferPower()
 {
 	//increase player power level and play sound
 	setPower(playerPower(CAM_HUMAN_PLAYER) + 4000);
-	playSound("power-transferred.ogg");
+	playSound(cam_sounds.powerTransferred);
 }
 
 function eventStartLevel()
 {
-	camSetStandardWinLossConditions(CAM_VICTORY_STANDARD, "SUB_2_2S");
+	camSetStandardWinLossConditions(CAM_VICTORY_STANDARD, cam_levels.beta4.pre);
 
 	const startPos = getObject("startPosition");
 	const lz = getObject("landingZone"); //player lz
 	centreView(startPos.x, startPos.y);
 	setNoGoArea(lz.x, lz.y, lz.x2, lz.y2, CAM_HUMAN_PLAYER);
 
-	const enemyLz = getObject("COLandingZone");
-	setNoGoArea(enemyLz.x, enemyLz.y, enemyLz.x2, enemyLz.y2, CAM_THE_COLLECTIVE);
-
 	setMissionTime(camChangeOnDiff(camHoursToSeconds(2)));
 	camPlayVideos([{video: "MB2_B_MSG", type: CAMP_MSG}, {video: "MB2_B_MSG2", type: MISS_MSG}]);
 
-	camSetArtifacts({
-		"COResearchLab": { tech: "R-Wpn-Flame2" },
-		"COHeavyFac-b4": { tech: "R-Wpn-RocketSlow-ROF01" },
-		"COHeavyFacL-b1": { tech: "R-Wpn-MG-ROF03" },
-		"COCommandCenter": { tech: "R-Vehicle-Body02" }, //Leopard
-		"COCybFac-b4": { tech: "R-Wpn-Cannon-ROF01" },
-		"COBombardPit": { tech: "R-Wpn-Mortar-Damage04" },
-	});
-
-	camCompleteRequiredResearch(mis_collectiveRes, CAM_THE_COLLECTIVE);
-
-	if (difficulty >= MEDIUM)
+	if (camClassicMode())
 	{
-		camUpgradeOnMapTemplates(cTempl.commc, cTempl.commrp, CAM_THE_COLLECTIVE);
-	}
+		camClassicResearch(mis_collectiveResClassic, CAM_THE_COLLECTIVE);
 
-	// New HMG Tiger Tracks units in first attack group
-	addDroid(CAM_THE_COLLECTIVE, 92, 59, "Heavy Machinegun Tiger Tracks", "Body9REC", "tracked01", "", "", "MG3Mk1");
-	addDroid(CAM_THE_COLLECTIVE, 96, 59, "Heavy Machinegun Tiger Tracks", "Body9REC", "tracked01", "", "", "MG3Mk1");
-	addDroid(CAM_THE_COLLECTIVE, 97, 59, "Heavy Machinegun Tiger Tracks", "Body9REC", "tracked01", "", "", "MG3Mk1");
+		camSetArtifacts({
+			"COResearchLab": { tech: "R-Wpn-Flame2" },
+			"COHeavyFac-b4": { tech: "R-Wpn-RocketSlow-ROF01" },
+			"COHeavyFacL-b1": { tech: "R-Wpn-MG-ROF03" },
+			"COCommandCenter": { tech: "R-Vehicle-Body06" }, //Panther
+		});
+	}
+	else
+	{
+		camCompleteRequiredResearch(mis_collectiveRes, CAM_THE_COLLECTIVE);
+
+		if (difficulty >= MEDIUM)
+		{
+			camUpgradeOnMapTemplates(cTempl.commc, cTempl.commrp, CAM_THE_COLLECTIVE);
+		}
+		// New HMG Tiger Tracks units in first attack group
+		if (difficulty >= HARD)
+		{
+			addDroid(CAM_THE_COLLECTIVE, 92, 59, "Heavy Machinegun Tiger Tracks", tBody.tank.tiger, tProp.tank.tracks, "", "", tWeap.tank.heavyMachinegun);
+			addDroid(CAM_THE_COLLECTIVE, 96, 59, "Heavy Machinegun Tiger Tracks", tBody.tank.tiger, tProp.tank.tracks, "", "", tWeap.tank.heavyMachinegun);
+			addDroid(CAM_THE_COLLECTIVE, 97, 59, "Heavy Machinegun Tiger Tracks", tBody.tank.tiger, tProp.tank.tracks, "", "", tWeap.tank.heavyMachinegun);
+		}
+		camUpgradeOnMapTemplates(cTempl.npcybf, cTempl.cocybth, CAM_THE_COLLECTIVE);
+
+		camSetArtifacts({
+			"COResearchLab": { tech: ["R-Wpn-Flame2", "R-Defense-WallUpgrade05"] },
+			"COHeavyFac-b4": { tech: "R-Wpn-RocketSlow-ROF01" },
+			"COHeavyFacL-b1": { tech: "R-Wpn-MG-ROF03" },
+			"COCommandCenter": { tech: "R-Vehicle-Body02" }, //Leopard
+			"COCybFac-b4": { tech: "R-Wpn-Cannon-ROF01" },
+			"COBombardPit": { tech: "R-Wpn-Mortar-Damage04" },
+		});
+	}
 
 	camSetEnemyBases({
 		"CONorthBase": {
 			cleanup: "base1Cleanup",
 			detectMsg: "C2B_BASE1",
-			detectSnd: "pcv379.ogg",
-			eliminateSnd: "pcv394.ogg",
+			detectSnd: cam_sounds.baseDetection.enemyBaseDetected,
+			eliminateSnd: cam_sounds.baseElimination.enemyBaseEradicated,
 		},
 		"COCentralBase": {
 			cleanup: "base2Cleanup",
 			detectMsg: "C2B_BASE2",
-			detectSnd: "pcv379.ogg",
-			eliminateSnd: "pcv394.ogg",
+			detectSnd: cam_sounds.baseDetection.enemyBaseDetected,
+			eliminateSnd: cam_sounds.baseElimination.enemyBaseEradicated,
 		},
 		"COMiddleBase": {
 			cleanup: "base4Cleanup",
 			detectMsg: "C2B_BASE4",
-			detectSnd: "pcv379.ogg",
-			eliminateSnd: "pcv394.ogg",
+			detectSnd: cam_sounds.baseDetection.enemyBaseDetected,
+			eliminateSnd: cam_sounds.baseElimination.enemyBaseEradicated,
 		},
 	});
 
@@ -180,7 +242,7 @@ function eventStartLevel()
 				repair: 30,
 				count: -1,
 			},
-			templates: [cTempl.comatt, cTempl.cohct, cTempl.commrp]
+			templates: (!camClassicMode()) ? [cTempl.comatt, cTempl.cohct, cTempl.commrp] : [cTempl.comatt, cTempl.cohct, cTempl.commc]
 		},
 		"COHeavyFacR-b1": {
 			assembly: "COHeavyFacR-b1Assembly",
@@ -192,7 +254,7 @@ function eventStartLevel()
 				repair: 30,
 				count: -1,
 			},
-			templates: [cTempl.comatt, cTempl.cohct, cTempl.commrp]
+			templates: (!camClassicMode()) ? [cTempl.comatt, cTempl.cohct, cTempl.commrp] : [cTempl.comatt, cTempl.cohct, cTempl.commc]
 		},
 		"COCybFacL-b2": {
 			assembly: "COCybFacL-b2Assembly",
@@ -216,7 +278,7 @@ function eventStartLevel()
 				repair: 40,
 				count: -1,
 			},
-			templates: [cTempl.npcybc, cTempl.npcybr, cTempl.npcybf, cTempl.npcybm]
+			templates: (!camClassicMode()) ? [cTempl.npcybc, cTempl.npcybr, cTempl.cocybth, cTempl.npcybm] : [cTempl.npcybc, cTempl.npcybr, cTempl.npcybf, cTempl.npcybm]
 		},
 		"COHeavyFac-b4": {
 			assembly: "COHeavyFac-b4Assembly",
@@ -240,7 +302,7 @@ function eventStartLevel()
 				repair: 40,
 				count: -1,
 			},
-			templates: [cTempl.npcybc, cTempl.npcybr, cTempl.npcybf]
+			templates: (!camClassicMode()) ? [cTempl.npcybc, cTempl.npcybr, cTempl.cocybth] : [cTempl.npcybc, cTempl.npcybr, cTempl.npcybf]
 		},
 	});
 
