@@ -104,11 +104,16 @@ namespace gfx_api
 }
 
 namespace WZ_vk {
-	using UniqueBuffer = vk::UniqueHandle<vk::Buffer, vk::DispatchLoaderDynamic>;
-	using UniqueDeviceMemory = vk::UniqueHandle<vk::DeviceMemory, vk::DispatchLoaderDynamic>;
-	using UniqueImage = vk::UniqueHandle<vk::Image, vk::DispatchLoaderDynamic>;
-	using UniqueImageView = vk::UniqueHandle<vk::ImageView, vk::DispatchLoaderDynamic>;
-	using UniqueSemaphore = vk::UniqueHandle<vk::Semaphore, vk::DispatchLoaderDynamic>;
+#if VK_HEADER_VERSION >= 301
+	using DispatchLoaderDynamic = vk::detail::DispatchLoaderDynamic;
+#else
+	using DispatchLoaderDynamic = vk::DispatchLoaderDynamic;
+#endif
+	using UniqueBuffer = vk::UniqueHandle<vk::Buffer, WZ_vk::DispatchLoaderDynamic>;
+	using UniqueDeviceMemory = vk::UniqueHandle<vk::DeviceMemory, WZ_vk::DispatchLoaderDynamic>;
+	using UniqueImage = vk::UniqueHandle<vk::Image, WZ_vk::DispatchLoaderDynamic>;
+	using UniqueImageView = vk::UniqueHandle<vk::ImageView, WZ_vk::DispatchLoaderDynamic>;
+	using UniqueSemaphore = vk::UniqueHandle<vk::Semaphore, WZ_vk::DispatchLoaderDynamic>;
 }
 
 inline void hash_combine(std::size_t& seed) { }
@@ -222,7 +227,7 @@ struct perFrameResources_t
 		{
 			pools.push_back(pool);
 		}
-		void reset(vk::Device dev, const vk::DispatchLoaderDynamic& vkDynLoader);
+		void reset(vk::Device dev, const WZ_vk::DispatchLoaderDynamic& vkDynLoader);
 		inline DescriptorPoolDetails& current() { return pools.at(currPool); }
 		bool nextPool() { if (!pools.empty() && (currPool < (pools.size() - 1))) { ++currPool; return true; } else { return false; } }
 	public:
@@ -252,7 +257,7 @@ struct perFrameResources_t
 	perFrameResources_t( const perFrameResources_t& other ) = delete; // non construction-copyable
 	perFrameResources_t& operator=( const perFrameResources_t& ) = delete; // non copyable
 
-	perFrameResources_t(vk::Device& _dev, const VmaAllocator& allocator, const uint32_t& graphicsQueueIndex, const vk::DispatchLoaderDynamic& vkDynLoader);
+	perFrameResources_t(vk::Device& _dev, const VmaAllocator& allocator, const uint32_t& graphicsQueueIndex, const WZ_vk::DispatchLoaderDynamic& vkDynLoader);
 	~perFrameResources_t();
 
 public:
@@ -295,7 +300,7 @@ private:
 	DescriptorPoolDetails createNewDescriptorPool(vk::DescriptorType type, uint32_t maxSets, uint32_t descriptorCount);
 
 private:
-	const vk::DispatchLoaderDynamic *pVkDynLoader;
+	const WZ_vk::DispatchLoaderDynamic *pVkDynLoader;
 	vk::CommandBuffer *pCurrentDrawCmdBuffer = nullptr;
 };
 
@@ -308,7 +313,7 @@ struct perSwapchainImageResources_t
 	perSwapchainImageResources_t( const perSwapchainImageResources_t& other ) = delete; // non construction-copyable
 	perSwapchainImageResources_t& operator=( const perSwapchainImageResources_t& ) = delete; // non copyable
 
-	perSwapchainImageResources_t(vk::Device& _dev, const vk::DispatchLoaderDynamic& vkDynLoader);
+	perSwapchainImageResources_t(vk::Device& _dev, const WZ_vk::DispatchLoaderDynamic& vkDynLoader);
 	~perSwapchainImageResources_t();
 
 protected:
@@ -316,7 +321,7 @@ protected:
 	void clean();
 
 private:
-	const vk::DispatchLoaderDynamic *pVkDynLoader;
+	const WZ_vk::DispatchLoaderDynamic *pVkDynLoader;
 };
 
 struct buffering_mechanism
@@ -329,9 +334,9 @@ struct buffering_mechanism
 
 	static perFrameResources_t& get_current_resources();
 	static perSwapchainImageResources_t& get_current_swapchain_resources();
-	static void init(vk::Device dev, const VmaAllocator& allocator, size_t swapchainImageCount, const uint32_t& graphicsQueueFamilyIndex, const vk::DispatchLoaderDynamic& vkDynLoader);
-	static void destroy(vk::Device dev, const vk::DispatchLoaderDynamic& vkDynLoader);
-	static void swap(vk::Device dev, const vk::DispatchLoaderDynamic& vkDynLoader, bool skipAcquireNewSwapchainImage);
+	static void init(vk::Device dev, const VmaAllocator& allocator, size_t swapchainImageCount, const uint32_t& graphicsQueueFamilyIndex, const WZ_vk::DispatchLoaderDynamic& vkDynLoader);
+	static void destroy(vk::Device dev, const WZ_vk::DispatchLoaderDynamic& vkDynLoader);
+	static void swap(vk::Device dev, const WZ_vk::DispatchLoaderDynamic& vkDynLoader, bool skipAcquireNewSwapchainImage);
 	static bool isInitialized();
 	static size_t get_current_frame_num();
 	static size_t numFrames();
@@ -369,17 +374,18 @@ struct VkPSO final
 	vk::ShaderModule vertexShader;
 	vk::ShaderModule fragmentShader;
 	vk::Device dev;
-	const vk::DispatchLoaderDynamic* pVkDynLoader;
+	const WZ_vk::DispatchLoaderDynamic* pVkDynLoader;
 	std::vector<vk::Sampler> samplers;
 
 	std::shared_ptr<VkhRenderPassCompat> renderpass_compat;
 	bool hasSpecializationConstant_ShadowConstants = false;
+	bool hasSpecializationConstant_PointLightConstants = false;
 
 private:
 	// Read shader into text buffer
 	static std::vector<uint32_t> readShaderBuf(const std::string& name);
 
-	vk::ShaderModule get_module(const std::string& name, const vk::DispatchLoaderDynamic& vkDynLoader);
+	vk::ShaderModule get_module(const std::string& name, const WZ_vk::DispatchLoaderDynamic& vkDynLoader);
 
 	static std::array<vk::PipelineShaderStageCreateInfo, 2> get_stages(const vk::ShaderModule& vertexModule, const vk::ShaderModule& fragmentModule);
 
@@ -404,7 +410,7 @@ public:
 		  vk::RenderPass rp,
 		  const std::shared_ptr<VkhRenderPassCompat>& renderpass_compat,
 		  vk::SampleCountFlagBits rasterizationSamples,
-		  const vk::DispatchLoaderDynamic& _vkDynLoader,
+		  const WZ_vk::DispatchLoaderDynamic& _vkDynLoader,
 		  const VkRoot& root
 		  );
 
@@ -458,6 +464,8 @@ struct VkTexture final : public gfx_api::texture
 	VmaAllocation allocation = VK_NULL_HANDLE;
 	gfx_api::pixel_format internal_format = gfx_api::pixel_format::invalid;
 	size_t mipmap_levels = 0;
+	size_t tex_width = 0;
+	size_t tex_height = 0;
 
 #if defined(WZ_DEBUG_GFX_API_LEAKS)
 	std::string debugName;
@@ -477,6 +485,7 @@ struct VkTexture final : public gfx_api::texture
 	virtual bool upload(const size_t& mip_level, const iV_BaseImage& image) override;
 	virtual bool upload_sub(const size_t& mip_level, const size_t& offset_x, const size_t& offset_y, const iV_Image& image) override;
 	virtual unsigned id() override;
+	virtual gfx_api::texture2dDimensions get_dimensions() const override;
 	virtual size_t backend_internal_value() const override;
 
 	VkTexture( const VkTexture& other ) = delete; // non construction-copyable
@@ -548,7 +557,7 @@ struct VkDepthMapImage final : public gfx_api::abstract_texture
 	virtual bool isArray() const override;
 	virtual size_t backend_internal_value() const override;
 
-	void destroy(vk::Device dev, const VmaAllocator& allocator, const vk::DispatchLoaderDynamic& vkDynLoader);
+	void destroy(vk::Device dev, const VmaAllocator& allocator, const WZ_vk::DispatchLoaderDynamic& vkDynLoader);
 
 	VkDepthMapImage( const VkDepthMapImage& other ) = delete; // non construction-copyable
 	VkDepthMapImage& operator=( const VkDepthMapImage& ) = delete; // non copyable
@@ -575,7 +584,7 @@ struct VkRenderedImage final : public gfx_api::abstract_texture
 	virtual bool isArray() const override;
 	virtual size_t backend_internal_value() const override;
 
-	void destroy(vk::Device dev, const VmaAllocator& allocator, const vk::DispatchLoaderDynamic& vkDynLoader);
+	void destroy(vk::Device dev, const VmaAllocator& allocator, const WZ_vk::DispatchLoaderDynamic& vkDynLoader);
 
 	VkRenderedImage( const VkRenderedImage& other ) = delete; // non construction-copyable
 	VkRenderedImage& operator=( const VkRenderedImage& ) = delete; // non copyable
@@ -605,7 +614,7 @@ struct VkRoot final : gfx_api::context
 	VkhInfo debugInfo;
 	gfx_api::context::swap_interval_mode swapMode = gfx_api::context::swap_interval_mode::vsync;
 	optional<float> mipLodBias;
-	gfx_api::shadow_constants shadowConstants;
+	gfx_api::lighting_constants shadowConstants;
 
 	std::vector<VkExtensionProperties> supportedInstanceExtensionProperties;
 	std::vector<const char*> instanceExtensions;
@@ -613,7 +622,7 @@ struct VkRoot final : gfx_api::context
 	vk::InstanceCreateInfo instanceCreateInfo;
 	vk::Instance inst;
 	std::vector<const char*> layers;
-	vk::DispatchLoaderDynamic vkDynLoader;
+	WZ_vk::DispatchLoaderDynamic vkDynLoader;
 
 	// physical device (and info)
 	vk::PhysicalDevice physicalDevice;
@@ -767,6 +776,9 @@ private:
 
 	bool getSupportedInstanceExtensions(std::vector<VkExtensionProperties> &output, PFN_vkGetInstanceProcAddr _vkGetInstanceProcAddr);
 	bool findSupportedInstanceExtensions(std::vector<const char*> extensionsToFind, std::vector<const char*> &output, PFN_vkGetInstanceProcAddr _vkGetInstanceProcAddr);
+#if defined(VK_EXT_layer_settings)
+	std::vector<vk::LayerSettingEXT> initLayerSettings();
+#endif
 	bool createVulkanInstance(uint32_t apiVersion, const std::vector<const char*>& extensions, const std::vector<const char*>& layers, PFN_vkGetInstanceProcAddr _vkGetInstanceProcAddr);
 	bool setupDebugUtilsCallbacks(const std::vector<const char*>& extensions, PFN_vkGetInstanceProcAddr _vkGetInstanceProcAddr);
 	bool setupDebugReportCallbacks(const std::vector<const char*>& extensions, PFN_vkGetInstanceProcAddr _vkGetInstanceProcAddr);
@@ -857,6 +869,7 @@ public:
 	virtual bool getScreenshot(std::function<void (std::unique_ptr<iV_Image>)> callback) override;
 	virtual void handleWindowSizeChange(unsigned int oldWidth, unsigned int oldHeight, unsigned int newWidth, unsigned int newHeight) override;
 	virtual std::pair<uint32_t, uint32_t> getDrawableDimensions() override;
+	bool isYAxisInverted() const override { return true; }
 	virtual bool shouldDraw() override;
 	virtual void shutdown() override;
 	virtual const size_t& current_FrameNum() const override;
@@ -867,12 +880,14 @@ public:
 	virtual bool supports2DTextureArrays() const override;
 	virtual bool supportsIntVertexAttributes() const override;
 	virtual size_t maxFramesInFlight() const override;
-	virtual gfx_api::shadow_constants getShadowConstants() override;
-	virtual bool setShadowConstants(gfx_api::shadow_constants values) override;
+	virtual gfx_api::lighting_constants getShadowConstants() override;
+	virtual bool setShadowConstants(gfx_api::lighting_constants values) override;
 	// instanced rendering APIs
 	virtual bool supportsInstancedRendering() override;
 	virtual void draw_instanced(const std::size_t& offset, const std::size_t &count, const gfx_api::primitive_type &primitive, std::size_t instance_count) override;
 	virtual void draw_elements_instanced(const std::size_t& offset, const std::size_t& count, const gfx_api::primitive_type& primitive, const gfx_api::index_type& index, std::size_t instance_count) override;
+	// debug apis for recompiling pipelines
+	virtual bool debugRecompileAllPipelines() override;
 private:
 	virtual bool _initialize(const gfx_api::backend_Impl_Factory& impl, int32_t antialiasing, swap_interval_mode mode, optional<float> mipLodBias, uint32_t depthMapResolution) override;
 	void initPixelFormatsSupport();
