@@ -73,7 +73,7 @@ W_CLICKFORM::W_CLICKFORM()
 	, AudioCallback(WidgGetAudioCallback())
 {}
 
-unsigned W_CLICKFORM::getState()
+unsigned W_CLICKFORM::getState() const
 {
 	return state & (WBUT_DISABLE | WBUT_LOCK | WBUT_CLICKLOCK | WBUT_FLASH | WBUT_DOWN | WBUT_HIGHLIGHT);
 }
@@ -163,15 +163,23 @@ void W_FORM::released(W_CONTEXT *psContext, WIDGET_KEY)
 
 void W_FORM::run(W_CONTEXT *psContext)
 {
-	if (!userMovable || !dragStart.has_value()) { return; }
+	// currently, no-op
+}
 
-	/* If the mouse is released *anywhere*, stop dragging */
-	if (mouseReleased(MOUSE_LMB))
+bool W_FORM::capturesMouseDrag(WIDGET_KEY wkey)
+{
+	return isUserMovable() && (wkey == WKEY_PRIMARY) && dragStart.has_value();
+}
+
+void W_FORM::mouseDragged(WIDGET_KEY wkey, W_CONTEXT *psStartContext, W_CONTEXT *psContext)
+{
+	if (wkey != WKEY_PRIMARY)
 	{
-		dragStart = nullopt;
 		return;
 	}
 
+	if (!userMovable || !dragStart.has_value()) { return; }
+
 	Vector2i currentMousePos(psContext->mx, psContext->my);
 	if (currentMousePos == dragStart.value()) { return; }
 	Vector2i dragDelta(currentMousePos.x - dragStart.value().x, currentMousePos.y - dragStart.value().y);
@@ -300,7 +308,7 @@ void W_FORM::screenSizeDidChange(int oldWidth, int oldHeight, int newWidth, int
 	WIDGET::screenSizeDidChange(oldWidth, oldHeight, newWidth, newHeight);
 }
 
-bool W_FORM::hitTest(int x, int y)
+bool W_FORM::hitTest(int x, int y) const
 {
 	if (!minimizable || formState != FormState::MINIMIZED)
 	{
