@@ -25,6 +25,7 @@
 #define __INCLUDED_MOVEDEF_H__
 
 #include "lib/framework/vector.h"
+#include "formationdef.h"
 
 #include <vector>
 
@@ -61,6 +62,8 @@ struct MOVE_CONTROL
 
 	unsigned shuffleStart = 0;            ///< When a shuffle started
 
+	FORMATION *psFormation = nullptr;     ///< formation the droid is currently a member of
+
 	int iVertSpeed = 0;                   ///< VTOL movement
 };
 
