@@ -1,6 +1,6 @@
 /*
 	This file is part of Warzone 2100.
-	Copyright (C) 2020-2021  Warzone 2100 Project
+	Copyright (C) 2020-2024  Warzone 2100 Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include "multistat.h"
 #include "multilobbycommands.h"
 #include "clparse.h"
+#include "main.h"
 
 #include <string>
 #include <atomic>
@@ -109,7 +110,7 @@ int quitSignalEventFd = -1;
 int quitSignalPipeFds[2] = {-1, -1};
 #endif
 
-#if !defined(_WIN32) && (defined(HAVE_SYS_EVENTFD_H) || defined(HAVE_UNISTD_H))
+#if !defined(_WIN32) && !defined(__EMSCRIPTEN__) && (defined(HAVE_SYS_EVENTFD_H) || defined(HAVE_UNISTD_H))
 # define WZ_STDIN_READER_SUPPORTED
 #endif
 
@@ -578,7 +579,7 @@ static bool changeHostChatPermissionsForActivePlayerWithIdentity(const std::stri
 		std::string playerVerifiedStatus = (ingame.VerifiedIdentity[i]) ? "V" : "?";
 		std::string playerName = NetPlay.players[i].name;
 		std::string playerNameB64 = base64Encode(std::vector<unsigned char>(playerName.begin(), playerName.end()));
-		wz_command_interface_output("WZEVENT: hostChatPermissions=%s: %" PRIu32 " %" PRIu32 "%s %s %s %s %s\n", (freeChatEnabled) ? "Y" : "N", i, gameTime, playerPublicKeyB64.c_str(), playerIdentityHash.c_str(), playerVerifiedStatus.c_str(), playerNameB64.c_str(), NetPlay.players[i].IPtextAddress);
+		wz_command_interface_output("WZEVENT: hostChatPermissions=%s: %" PRIu32 " %" PRIu32 " %s %s %s %s %s\n", (freeChatEnabled) ? "Y" : "N", i, gameTime, playerPublicKeyB64.c_str(), playerIdentityHash.c_str(), playerVerifiedStatus.c_str(), playerNameB64.c_str(), NetPlay.players[i].IPtextAddress);
 	});
 
 	if (result)
@@ -755,7 +756,7 @@ int cmdInputThreadFunc(void *)
 					bool foundActivePlayer = kickActivePlayerWithIdentity(playerIdentityStrCopy, kickReasonStrCopy, false);
 					if (!foundActivePlayer)
 					{
-						wz_command_interface_output("WZCMD error: Failed to find currently-connected player with matching public key or hash?\n");
+						wz_command_interface_output("WZCMD info: kick identity %s: failed to find currently-connected player with matching public key or hash\n", playerIdentityStrCopy.c_str());
 					}
 				});
 			}
@@ -841,11 +842,12 @@ int cmdInputThreadFunc(void *)
 					continue;
 				}
 				std::string playerIdentityStrCopy(playeridentitystring);
-				wzAsyncExecOnMainThread([playerIdentityStrCopy, freeChatEnabled] {
+				std::string chatLevelStrCopy(chatlevel);
+				wzAsyncExecOnMainThread([playerIdentityStrCopy, chatLevelStrCopy, freeChatEnabled] {
 					bool foundActivePlayer = changeHostChatPermissionsForActivePlayerWithIdentity(playerIdentityStrCopy, freeChatEnabled);
 					if (!foundActivePlayer)
 					{
-						wz_command_interface_output("WZCMD error: Failed to find currently-connected player with matching public key or hash?\n");
+						wz_command_interface_output("WZCMD info: set chat %s %s: failed to find currently-connected player with matching public key or hash\n", chatLevelStrCopy.c_str(), playerIdentityStrCopy.c_str());
 					}
 				});
 			}
@@ -997,7 +999,7 @@ int cmdInputThreadFunc(void *)
 					}
 					if (!foundActivePlayer)
 					{
-						wz_command_interface_output("WZCMD error: Failed to find currently-connected player with matching public key or hash?\n");
+						wz_command_interface_output("WZCMD info: chat direct %s: failed to find currently-connected player with matching public key or hash\n", playerIdentityStrCopy.c_str());
 					}
 				});
 			}
@@ -1006,29 +1008,36 @@ int cmdInputThreadFunc(void *)
 		{
 			char action[1024] = {0};
 			char uniqueJoinID[1024] = {0};
+			char rejectionMessage[MAX_JOIN_REJECT_REASON] = {0};
 			unsigned int rejectionReason = static_cast<unsigned int>(ERROR_NOERROR);
-			int r = sscanf(line, "join %1023s %1023s %u", action, uniqueJoinID, &rejectionReason);
-			if (r != 2 && r != 3)
+			int r = sscanf(line, "join %1023s %1023s %u %2047[^\n]s", action, uniqueJoinID, &rejectionReason, rejectionMessage);
+			if (r < 2 || r > 4)
 			{
-				wz_command_interface_output_onmainthread("WZCMD error: Failed to get join action or uniqueJoinID!\n");
+				wz_command_interface_output_onmainthread("WZCMD error: Failed to parse join command!\n");
 			}
 			else
 			{
-				optional<bool> approve = nullopt;
+				optional<AsyncJoinApprovalAction> approve = nullopt;
 				if (strcmp(action, "approve") == 0)
 				{
-					approve = true;
+					approve = AsyncJoinApprovalAction::Approve;
 				}
 				else if (strcmp(action, "reject") == 0)
 				{
-					approve = false;
+					approve = AsyncJoinApprovalAction::Reject;
 				}
-				if (approve.has_value() || rejectionReason > static_cast<unsigned int>(std::numeric_limits<uint8_t>::max()))
+				else if (strcmp(action, "approvespec") == 0)
 				{
-					bool approveValue = approve.value();
+					approve = AsyncJoinApprovalAction::ApproveSpectators;
+				}
+				if (approve.has_value() && rejectionReason < static_cast<unsigned int>(std::numeric_limits<uint8_t>::max()))
+				{
+					auto approveValue = approve.value();
 					std::string uniqueJoinIDCopy(uniqueJoinID);
-					wzAsyncExecOnMainThread([uniqueJoinIDCopy, approveValue, rejectionReason] {
-						if (!NETsetAsyncJoinApprovalResult(uniqueJoinIDCopy, approveValue, static_cast<LOBBY_ERROR_TYPES>(rejectionReason)))
+					std::string rejectionMessageCopy(rejectionMessage);
+					convertEscapedNewlines(rejectionMessageCopy);
+					wzAsyncExecOnMainThread([uniqueJoinIDCopy, approveValue, rejectionReason, rejectionMessageCopy] {
+						if (!NETsetAsyncJoinApprovalResult(uniqueJoinIDCopy, approveValue, static_cast<LOBBY_ERROR_TYPES>(rejectionReason), rejectionMessageCopy))
 						{
 							wz_command_interface_output("WZCMD info: Could not find currently-waiting join with specified uniqueJoinID\n");
 						}
@@ -1040,6 +1049,25 @@ int cmdInputThreadFunc(void *)
 				}
 			}
 		}
+		else if(!strncmpl(line, "autobalance"))
+		{
+			wzAsyncExecOnMainThread([] {
+				if (autoBalancePlayersCmd())
+				{
+					wz_command_interface_output("WZCMD info: autobalanced players\n");
+				}
+				else
+				{
+					wz_command_interface_output("WZCMD error: autobalance failed\n");
+				}
+			});
+		}
+		else if(!strncmpl(line, "status"))
+		{
+			wzAsyncExecOnMainThread([] {
+				wz_command_interface_output_room_status_json();
+			});
+		}
 		else if(!strncmpl(line, "shutdown now"))
 		{
 			inexit = true;
@@ -1430,3 +1458,204 @@ void configSetCmdInterface(WZ_Command_Interface mode, std::string value)
 	}
 	wz_cmd_interface_param = value;
 }
+
+// MARK: - Output Room Status JSON
+
+static void WzCmdInterfaceDumpHumanPlayerVarsImpl(uint32_t player, bool gameHasFiredUp, nlohmann::ordered_json& j)
+{
+	PLAYER const &p = NetPlay.players[player];
+
+	j["name"] = p.name;
+
+	if (!gameHasFiredUp)
+	{
+		// in lobby, output "ready" status
+		j["ready"] = static_cast<int>(p.ready);
+	}
+	else
+	{
+		// once game has fired up, output loading / connection status
+		if (p.allocated)
+		{
+			if (ingame.JoiningInProgress[player])
+			{
+				j["status"] = "loading";
+			}
+			else
+			{
+				j["status"] = "active";
+			}
+			if (ingame.PendingDisconnect[player])
+			{
+				j["status"] = "pendingleave";
+			}
+		}
+		else
+		{
+			j["status"] = "left";
+		}
+	}
+
+	const auto& identity = getMultiStats(player).identity;
+	if (!identity.empty())
+	{
+		j["pk"] = base64Encode(identity.toBytes(EcKey::Public));
+	}
+	else
+	{
+		j["pk"] = "";
+	}
+	j["ip"] = NetPlay.players[player].IPtextAddress;
+
+	if (ingame.PingTimes[player] != PING_LIMIT)
+	{
+		j["ping"] = ingame.PingTimes[player];
+	}
+	else
+	{
+		j["ping"] = -1; // for "infinite" ping
+	}
+
+	j["admin"] = static_cast<int>(NetPlay.players[player].isAdmin || (player == NetPlay.hostPlayer));
+
+	if (player == NetPlay.hostPlayer)
+	{
+		j["host"] = 1;
+	}
+}
+
+void wz_command_interface_output_room_status_json()
+{
+	if (!wz_command_interface_enabled())
+	{
+		return;
+	}
+
+	bool gameHasFiredUp = (GetGameMode() == GS_NORMAL);
+
+	auto root = nlohmann::ordered_json::object();
+	root["ver"] = 1;
+
+	auto data = nlohmann::ordered_json::object();
+	if (gameHasFiredUp)
+	{
+		if (ingame.TimeEveryoneIsInGame.has_value())
+		{
+			data["state"] = "started";
+		}
+		else
+		{
+			data["state"] = "starting";
+		}
+	}
+	else
+	{
+		data["state"] = "lobby";
+	}
+	if (NetPlay.isHost)
+	{
+		auto lobbyGameId = NET_getCurrentHostedLobbyGameId();
+		if (lobbyGameId != 0)
+		{
+			data["lobbyid"] = lobbyGameId;
+		}
+	}
+	data["map"] = game.map;
+
+	root["data"] = std::move(data);
+
+	if (NetPlay.isHost)
+	{
+		auto players = nlohmann::ordered_json::array();
+		for (uint8_t player = 0; player < game.maxPlayers; ++player)
+		{
+			PLAYER const &p = NetPlay.players[player];
+			auto j = nlohmann::ordered_json::object();
+
+			j["pos"] = p.position;
+			j["team"] = p.team;
+			j["col"] = p.colour;
+			j["fact"] = static_cast<int32_t>(p.faction);
+
+			if (p.ai == AI_CLOSED)
+			{
+				// closed slot
+				j["type"] = "closed";
+			}
+			else if (p.ai == AI_OPEN)
+			{
+				if (!gameHasFiredUp && !p.allocated)
+				{
+					// available / open slot (in lobby)
+					j["type"] = "open";
+				}
+				else
+				{
+					if (!p.allocated)
+					{
+						// if game has fired up and this slot is no longer allocated, skip it entirely if it wasn't initially a human player
+						if (p.difficulty != AIDifficulty::HUMAN)
+						{
+							continue;
+						}
+					}
+
+					// human (or host) slot
+					j["type"] = (p.isSpectator) ? "spec" : "player";
+
+					WzCmdInterfaceDumpHumanPlayerVarsImpl(player, gameHasFiredUp, j);
+				}
+			}
+			else
+			{
+				// bot player
+				j["type"] = "bot";
+
+				j["name"] = getAIName(player);
+				j["difficulty"] = static_cast<int>(NetPlay.players[player].difficulty);
+			}
+
+			players.push_back(std::move(j));
+		}
+		root["players"] = std::move(players);
+
+		auto spectators = nlohmann::ordered_json::array();
+		for (uint32_t i = MAX_PLAYER_SLOTS; i < MAX_CONNECTED_PLAYERS; ++i)
+		{
+			PLAYER const &p = NetPlay.players[i];
+			if (p.ai == AI_CLOSED)
+			{
+				continue;
+			}
+
+			auto j = nlohmann::ordered_json::object();
+			if (!p.allocated)
+			{
+				if (!gameHasFiredUp)
+				{
+					// available / open spectator slot
+					j["type"] = "open";
+				}
+				else
+				{
+					// no spectator connected to this slot - skip
+					continue;
+				}
+			}
+			else
+			{
+				// human (or host) slot
+				j["type"] = (p.isSpectator) ? "spec" : "player";
+
+				WzCmdInterfaceDumpHumanPlayerVarsImpl(i, gameHasFiredUp, j);
+			}
+
+			spectators.push_back(std::move(j));
+		}
+		root["specs"] = std::move(spectators);
+	}
+
+	std::string statusJSONStr = std::string("__WZROOMSTATUS__") + root.dump(-1, ' ', false, nlohmann::ordered_json::error_handler_t::replace) + "__ENDWZROOMSTATUS__";
+	statusJSONStr.append("\n");
+	wz_command_interface_output_str(statusJSONStr.c_str());
+}
